import pydata_google_auth
from google.cloud import bigquery


def pydata_auth():
    # https://pandas-gbq.readthedocs.io/en/latest/howto/authentication.html#authenticating-with-a-user-account
    SCOPES = [
        "https://www.googleapis.com/auth/cloud-platform",
        "https://www.googleapis.com/auth/drive",
    ]
    return pydata_google_auth.get_user_credentials(
        SCOPES,
        # Set auth_local_webserver to True to have a slightly more convienient
        # authorization flow. Note, this doesn't work if you're running from a
        # notebook on a remote sever, such as over SSH or with Google Colab.
        auth_local_webserver=True,
    )


def gcloud_auth():
    return bigquery.Client()



from dataclasses import dataclass
from pandas import DataFrame, read_gbq
from geopandas import GeoDataFrame, GeoSeries

from .auth import pydata_auth, gcloud_auth


@dataclass
class BQWarehouse:

    project_id: str
    credentials = None
    client = None

    @property
    def pydata_creds(self):
        if not self.credentials:
            self.credentials = pydata_auth()

        return self.credentials

    @property
    def gcloud_client(self):
        if not self.client:
            self.client = gcloud_auth()

        return self.client

    ## READ data from bigquery
    ## -----------------------

    def read_df(self, query: str, **kwargs) -> DataFrame:
        """
        Read `query` into a `pd.DataFrame` with
        any optional kwargs defined here:
        https://pandas.pydata.org/docs/reference/api/pandas.read_gbq.html

        Arguments:
            query: str

        Returns:
            pd.DataFrame
        """
        return read_gbq(
            query=query,
            project_id=self.project_id,
            credentials=self.pydata_creds,
            **kwargs,
        )

    def read_gdf(
        self,
        query: str,
        geom_col: str = "geom",
        read_kwargs: dict = {},
        gdf_kwargs: dict = {},
    ) -> GeoDataFrame:
        df = self.read_df(query=query, **read_kwargs)
        df[geom_col] = GeoSeries.from_wkt(df[geom_col])

        return GeoDataFrame(
            df,
            geometry=geom_col,
            **gdf_kwargs,
        )

    ## WRITE data to bigquery
    ## ----------------------

    def write_df(
        self,
        df: DataFrame,
        destination: str,
        **kwargs,
    ) -> None:
        job = self.gcloud_client.load_table_from_dataframe(
            df,
            destination=destination,
            **kwargs,
        )
        return job.result()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['warehouses', 'warehouses.bigquery']

package_data = \
{'': ['*']}

install_requires = \
['geopandas>=0.10.2,<0.11.0',
 'google-cloud-bigquery>=3.1.0,<4.0.0',
 'pandas-gbq>=0.17.4,<0.18.0',
 'tqdm>=4.64.0,<5.0.0']

setup_kwargs = {
    'name': 'warehouses',
    'version': '0.1.0',
    'description': 'Python library to facilitate read/write of GIS and tabular data between Python and cloud warehouses',
    'long_description': None,
    'author': 'aaronfraint',
    'author_email': 'aaronfraint@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
